'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true,
});
exports.HueSlider = HueSlider;
var _react = _interopRequireWildcard(require('react'));
var _reactNative = require('react-native');
var _reactNativeGestureHandler = require('react-native-gesture-handler');
var _reactNativeReanimated = _interopRequireWildcard(require('react-native-reanimated'));
var _utils = require('../../utils');
var _AppContext = _interopRequireDefault(require('../../AppContext'));
var _Thumb = _interopRequireDefault(require('../Thumb/Thumb'));
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}
function _getRequireWildcardCache(nodeInterop) {
  if (typeof WeakMap !== 'function') return null;
  var cacheBabelInterop = new WeakMap();
  var cacheNodeInterop = new WeakMap();
  return (_getRequireWildcardCache = function (nodeInterop) {
    return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
  })(nodeInterop);
}
function _interopRequireWildcard(obj, nodeInterop) {
  if (!nodeInterop && obj && obj.__esModule) {
    return obj;
  }
  if (obj === null || (typeof obj !== 'object' && typeof obj !== 'function')) {
    return { default: obj };
  }
  var cache = _getRequireWildcardCache(nodeInterop);
  if (cache && cache.has(obj)) {
    return cache.get(obj);
  }
  var newObj = {};
  var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var key in obj) {
    if (key !== 'default' && Object.prototype.hasOwnProperty.call(obj, key)) {
      var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
      if (desc && (desc.get || desc.set)) {
        Object.defineProperty(newObj, key, desc);
      } else {
        newObj[key] = obj[key];
      }
    }
  }
  newObj.default = obj;
  if (cache) {
    cache.set(obj, newObj);
  }
  return newObj;
}
function HueSlider(_ref) {
  let {
    adaptSpectrum: localAdaptSpectrum,
    thumbShape: localThumbShape,
    thumbSize: localThumbSize,
    thumbColor: localThumbColor,
    boundedThumb: localBoundedThumb,
    renderThumb: localRenderThumb,
    thumbInnerStyle: localThumbInnerStyle,
    thumbStyle: localThumbStyle,
    sliderThickness: localSliderThickness,
    style = {},
    vertical = false,
    reverse = false,
  } = _ref;
  const {
    hueValue,
    saturationValue,
    brightnessValue,
    onGestureChange,
    onGestureEnd,
    adaptSpectrum: globalAdaptSpectrum,
    thumbSize: globalThumbSize,
    thumbShape: globalThumbShape,
    thumbColor: globalThumbColor,
    boundedThumb: globalBoundedThumb,
    renderThumb: globalRenderThumb,
    thumbStyle: globalThumbStyle,
    thumbInnerStyle: globalThumbInnerStyle,
    sliderThickness: globalSliderThickness,
  } = (0, _react.useContext)(_AppContext.default);
  const thumbShape = localThumbShape ?? globalThumbShape,
    thumbSize = localThumbSize ?? globalThumbSize,
    thumbColor = localThumbColor ?? globalThumbColor,
    boundedThumb = localBoundedThumb ?? globalBoundedThumb,
    renderThumb = localRenderThumb ?? globalRenderThumb,
    thumbStyle = localThumbStyle ?? globalThumbStyle ?? {},
    thumbInnerStyle = localThumbInnerStyle ?? globalThumbInnerStyle ?? {},
    adaptSpectrum = localAdaptSpectrum ?? globalAdaptSpectrum,
    sliderThickness = localSliderThickness ?? globalSliderThickness;
  const borderRadius = (0, _utils.getStyle)(style, 'borderRadius') ?? 5,
    getWidth = (0, _utils.getStyle)(style, 'width'),
    getHeight = (0, _utils.getStyle)(style, 'height');
  const width = (0, _reactNativeReanimated.useSharedValue)(
      vertical ? sliderThickness : typeof getWidth === 'number' ? getWidth : 0
    ),
    height = (0, _reactNativeReanimated.useSharedValue)(
      !vertical ? sliderThickness : typeof getHeight === 'number' ? getHeight : 0
    );
  const handleScale = (0, _reactNativeReanimated.useSharedValue)(1);
  const handleStyle = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        const length = (vertical ? height.value : width.value) - (boundedThumb ? thumbSize : 0),
          percent = (hueValue.value / 360) * length,
          pos = (reverse ? length - percent : percent) - (boundedThumb ? 0 : thumbSize / 2),
          posY = vertical ? pos : height.value / 2 - thumbSize / 2,
          posX = vertical ? width.value / 2 - thumbSize / 2 : pos;
        return {
          transform: [
            {
              translateY: posY,
            },
            {
              translateX: posX,
            },
            {
              scale: handleScale.value,
            },
          ],
        };
      };
      _f._closure = {
        vertical,
        height,
        width,
        boundedThumb,
        thumbSize,
        hueValue,
        reverse,
        handleScale,
      };
      _f.asString =
        'function _f(){const{vertical,height,width,boundedThumb,thumbSize,hueValue,reverse,handleScale}=jsThis._closure;{const length=(vertical?height.value:width.value)-(boundedThumb?thumbSize:0),percent=hueValue.value/360*length,pos=(reverse?length-percent:percent)-(boundedThumb?0:thumbSize/2),posY=vertical?pos:height.value/2-thumbSize/2,posX=vertical?width.value/2-thumbSize/2:pos;return{transform:[{translateY:posY},{translateX:posX},{scale:handleScale.value}]};}}';
      _f.__workletHash = 11477396967598;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (64:39)';
      _f.__optimalization = 3;
      return _f;
    })(),
    [localThumbSize, vertical, reverse]
  );
  const activeSaturationStyle = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        if (!adaptSpectrum) return {};
        return {
          backgroundColor: (0, _utils.HSVA2HSLA)(0, 0, brightnessValue.value, 1 - saturationValue.value / 100),
        };
      };
      _f._closure = {
        adaptSpectrum,
        HSVA2HSLA: _utils.HSVA2HSLA,
        brightnessValue,
        saturationValue,
      };
      _f.asString =
        'function _f(){const{adaptSpectrum,HSVA2HSLA,brightnessValue,saturationValue}=jsThis._closure;{if(!adaptSpectrum)return{};return{backgroundColor:HSVA2HSLA(0,0,brightnessValue.value,1-saturationValue.value/100)};}}';
      _f.__workletHash = 8782818419074;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (75:49)';
      return _f;
    })()
  );
  const activeBrightnessStyle = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        if (!adaptSpectrum) return {};
        return {
          backgroundColor: (0, _utils.HSVA2HSLA)(0, 0, 0, 1 - brightnessValue.value / 100),
        };
      };
      _f._closure = {
        adaptSpectrum,
        HSVA2HSLA: _utils.HSVA2HSLA,
        brightnessValue,
      };
      _f.asString =
        'function _f(){const{adaptSpectrum,HSVA2HSLA,brightnessValue}=jsThis._closure;{if(!adaptSpectrum)return{};return{backgroundColor:HSVA2HSLA(0,0,0,1-brightnessValue.value/100)};}}';
      _f.__workletHash = 12941901269723;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (79:49)';
      return _f;
    })()
  );
  const onGestureUpdate = (function () {
    const _f = function (_ref2) {
      let { x, y } = _ref2;
      const length = (vertical ? height.value : width.value) - (boundedThumb ? thumbSize : 0),
        pos = (0, _utils.clamp)((vertical ? y : x) - (boundedThumb ? thumbSize / 2 : 0), length),
        value = Math.round((pos / length) * 360),
        newHueValue = reverse ? 360 - value : value;
      if (hueValue.value === newHueValue) return;
      hueValue.value = newHueValue;
      (0, _reactNativeReanimated.runOnJS)(onGestureChange)();
    };
    _f._closure = {
      vertical,
      height,
      width,
      boundedThumb,
      thumbSize,
      clamp: _utils.clamp,
      reverse,
      hueValue,
      runOnJS: _reactNativeReanimated.runOnJS,
      onGestureChange,
    };
    _f.asString =
      'function _f({x:x,y:y}){const{vertical,height,width,boundedThumb,thumbSize,clamp,reverse,hueValue,runOnJS,onGestureChange}=jsThis._closure;{const length=(vertical?height.value:width.value)-(boundedThumb?thumbSize:0),pos=clamp((vertical?y:x)-(boundedThumb?thumbSize/2:0),length),value=Math.round(pos/length*360),newHueValue=reverse?360-value:value;if(hueValue.value===newHueValue)return;hueValue.value=newHueValue;runOnJS(onGestureChange)();}}';
    _f.__workletHash = 17055761397986;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (84:26)';
    return _f;
  })();
  const onGestureBegin = (function () {
    const _f = function (event) {
      handleScale.value = (0, _reactNativeReanimated.withTiming)(1.2, {
        duration: 100,
      });
      onGestureUpdate(event);
    };
    _f._closure = {
      handleScale,
      withTiming: _reactNativeReanimated.withTiming,
      onGestureUpdate,
    };
    _f.asString =
      'function _f(event){const{handleScale,withTiming,onGestureUpdate}=jsThis._closure;{handleScale.value=withTiming(1.2,{duration:100});onGestureUpdate(event);}}';
    _f.__workletHash = 14773240202957;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (97:25)';
    return _f;
  })();
  const onGestureFinish = (function () {
    const _f = function () {
      handleScale.value = (0, _reactNativeReanimated.withTiming)(1, {
        duration: 100,
      });
      (0, _reactNativeReanimated.runOnJS)(onGestureEnd)();
    };
    _f._closure = {
      handleScale,
      withTiming: _reactNativeReanimated.withTiming,
      runOnJS: _reactNativeReanimated.runOnJS,
      onGestureEnd,
    };
    _f.asString =
      'function _f(){const{handleScale,withTiming,runOnJS,onGestureEnd}=jsThis._closure;{handleScale.value=withTiming(1,{duration:100});runOnJS(onGestureEnd)();}}';
    _f.__workletHash = 6379049506140;
    _f.__location =
      'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (102:26)';
    return _f;
  })();
  const pan = _reactNativeGestureHandler.Gesture.Pan().onBegin(onGestureBegin).onUpdate(onGestureUpdate).onEnd(onGestureFinish);
  const tap = _reactNativeGestureHandler.Gesture.Tap().onTouchesUp(onGestureFinish);
  const longPress = _reactNativeGestureHandler.Gesture.LongPress().onTouchesUp(onGestureFinish);
  const composed = _reactNativeGestureHandler.Gesture.Exclusive(pan, tap, longPress);
  const onLayout = _ref3 => {
    let {
      nativeEvent: { layout },
    } = _ref3;
    if (!vertical)
      width.value = (0, _reactNativeReanimated.withTiming)(layout.width, {
        duration: 5,
      });
    if (vertical)
      height.value = (0, _reactNativeReanimated.withTiming)(layout.height, {
        duration: 5,
      });
  };
  const imageStyle = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        const imageRotate = vertical ? (reverse ? '270deg' : '90deg') : reverse ? '180deg' : '0deg';
        const imageTranslateY =
          ((height.value - width.value) / 2) * ((reverse && _utils.isRtl) || (!reverse && !_utils.isRtl) ? 1 : -1);
        return {
          width: vertical ? height.value : '100%',
          height: vertical ? width.value : '100%',
          borderRadius,
          transform: [
            {
              rotate: imageRotate,
            },
            {
              translateX: vertical ? ((height.value - width.value) / 2) * (reverse ? -1 : 1) : 0,
            },
            {
              translateY: vertical ? imageTranslateY : 0,
            },
          ],
        };
      };
      _f._closure = {
        vertical,
        reverse,
        height,
        width,
        isRtl: _utils.isRtl,
        borderRadius,
      };
      _f.asString =
        "function _f(){const{vertical,reverse,height,width,isRtl,borderRadius}=jsThis._closure;{const imageRotate=vertical?reverse?'270deg':'90deg':reverse?'180deg':'0deg';const imageTranslateY=(height.value-width.value)/2*(reverse&&isRtl||!reverse&&!isRtl?1:-1);return{width:vertical?height.value:'100%',height:vertical?width.value:'100%',borderRadius:borderRadius,transform:[{rotate:imageRotate},{translateX:vertical?(height.value-width.value)/2*(reverse?-1:1):0},{translateY:vertical?imageTranslateY:0}]};}}";
      _f.__workletHash = 9330412360531;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\HueSlider.tsx (118:38)';
      _f.__optimalization = 3;
      return _f;
    })(),
    [vertical, reverse, sliderThickness]
  );
  const thicknessStyle = vertical
    ? {
        width: sliderThickness,
      }
    : {
        height: sliderThickness,
      };
  return /*#__PURE__*/ _react.default.createElement(
    _reactNativeGestureHandler.GestureDetector,
    {
      gesture: composed,
    },
    /*#__PURE__*/ _react.default.createElement(
      _reactNativeReanimated.default.View,
      {
        onLayout: onLayout,
        style: [
          {
            borderRadius,
          },
          style,
          thicknessStyle,
          {
            position: 'relative',
            borderWidth: 0,
            padding: 0,
          },
        ],
      },
      /*#__PURE__*/ _react.default.createElement(_reactNativeReanimated.default.Image, {
        source: require('../../assets/Hue.png'),
        style: imageStyle,
      }),
      /*#__PURE__*/ _react.default.createElement(
        _utils.ConditionalRendering,
        {
          render: adaptSpectrum,
        },
        /*#__PURE__*/ _react.default.createElement(_reactNativeReanimated.default.View, {
          style: [
            {
              borderRadius,
            },
            activeBrightnessStyle,
            _reactNative.StyleSheet.absoluteFillObject,
          ],
        }),
        /*#__PURE__*/ _react.default.createElement(_reactNativeReanimated.default.View, {
          style: [
            {
              borderRadius,
            },
            activeSaturationStyle,
            _reactNative.StyleSheet.absoluteFillObject,
          ],
        })
      ),
      /*#__PURE__*/ _react.default.createElement(_Thumb.default, {
        channel: 'h',
        thumbShape,
        thumbSize,
        thumbColor,
        renderThumb,
        handleStyle,
        innerStyle: thumbInnerStyle,
        style: thumbStyle,
        vertical,
        adaptSpectrum,
      })
    )
  );
}
//# sourceMappingURL=HueSlider.js.map
