'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true,
});
exports.BlueSlider = BlueSlider;
var _react = _interopRequireWildcard(require('react'));
var _reactNativeGestureHandler = require('react-native-gesture-handler');
var _reactNativeReanimated = _interopRequireWildcard(require('react-native-reanimated'));
var _utils = require('../../../utils');
var _AppContext = _interopRequireDefault(require('../../../AppContext'));
var _Thumb = _interopRequireDefault(require('../../Thumb/Thumb'));
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}
function _getRequireWildcardCache(nodeInterop) {
  if (typeof WeakMap !== 'function') return null;
  var cacheBabelInterop = new WeakMap();
  var cacheNodeInterop = new WeakMap();
  return (_getRequireWildcardCache = function (nodeInterop) {
    return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
  })(nodeInterop);
}
function _interopRequireWildcard(obj, nodeInterop) {
  if (!nodeInterop && obj && obj.__esModule) {
    return obj;
  }
  if (obj === null || (typeof obj !== 'object' && typeof obj !== 'function')) {
    return { default: obj };
  }
  var cache = _getRequireWildcardCache(nodeInterop);
  if (cache && cache.has(obj)) {
    return cache.get(obj);
  }
  var newObj = {};
  var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var key in obj) {
    if (key !== 'default' && Object.prototype.hasOwnProperty.call(obj, key)) {
      var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
      if (desc && (desc.get || desc.set)) {
        Object.defineProperty(newObj, key, desc);
      } else {
        newObj[key] = obj[key];
      }
    }
  }
  newObj.default = obj;
  if (cache) {
    cache.set(obj, newObj);
  }
  return newObj;
}
function BlueSlider(_ref) {
  let {
    thumbShape: localThumbShape,
    thumbSize: localThumbSize,
    thumbColor: localThumbColor,
    boundedThumb: localBoundedThumb,
    renderThumb: localRenderThumb,
    thumbStyle: localThumbStyle,
    thumbInnerStyle: localThumbInnerStyle,
    sliderThickness: localSliderThickness,
    style = {},
    vertical = false,
    reverse = false,
  } = _ref;
  const {
    hueValue,
    saturationValue,
    brightnessValue,
    onGestureChange,
    onGestureEnd,
    thumbSize: globalThumbSize,
    thumbShape: globalThumbShape,
    thumbColor: globalThumbColor,
    boundedThumb: globalBoundedThumb,
    renderThumb: globalRenderThumb,
    thumbStyle: globalThumbStyle,
    thumbInnerStyle: globalThumbInnerStyle,
    sliderThickness: globalSliderThickness,
  } = (0, _react.useContext)(_AppContext.default);
  const thumbShape = localThumbShape ?? globalThumbShape,
    thumbSize = localThumbSize ?? globalThumbSize,
    thumbColor = localThumbColor ?? globalThumbColor,
    boundedThumb = localBoundedThumb ?? globalBoundedThumb,
    renderThumb = localRenderThumb ?? globalRenderThumb,
    thumbStyle = localThumbStyle ?? globalThumbStyle ?? {},
    thumbInnerStyle = localThumbInnerStyle ?? globalThumbInnerStyle ?? {},
    sliderThickness = localSliderThickness ?? globalSliderThickness;
  const borderRadius = (0, _utils.getStyle)(style, 'borderRadius') ?? 5,
    getWidth = (0, _utils.getStyle)(style, 'width'),
    getHeight = (0, _utils.getStyle)(style, 'height');
  const width = (0, _reactNativeReanimated.useSharedValue)(
      vertical ? sliderThickness : typeof getWidth === 'number' ? getWidth : 0
    ),
    height = (0, _reactNativeReanimated.useSharedValue)(
      !vertical ? sliderThickness : typeof getHeight === 'number' ? getHeight : 0
    );
  const handleScale = (0, _reactNativeReanimated.useSharedValue)(1);
  const handleStyle = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        const { b } = (0, _utils.HSVA2RGBA)(hueValue.value, saturationValue.value, brightnessValue.value);
        const length = (vertical ? height.value : width.value) - (boundedThumb ? thumbSize : 0),
          percent = (b / 255) * length,
          pos = (reverse ? length - percent : percent) - (boundedThumb ? 0 : thumbSize / 2),
          posY = vertical ? pos : height.value / 2 - thumbSize / 2,
          posX = vertical ? width.value / 2 - thumbSize / 2 : pos;
        return {
          transform: [
            {
              translateY: posY,
            },
            {
              translateX: posX,
            },
            {
              scale: handleScale.value,
            },
          ],
        };
      };
      _f._closure = {
        HSVA2RGBA: _utils.HSVA2RGBA,
        hueValue,
        saturationValue,
        brightnessValue,
        vertical,
        height,
        width,
        boundedThumb,
        thumbSize,
        reverse,
        handleScale,
      };
      _f.asString =
        'function _f(){const{HSVA2RGBA,hueValue,saturationValue,brightnessValue,vertical,height,width,boundedThumb,thumbSize,reverse,handleScale}=jsThis._closure;{const{b:b}=HSVA2RGBA(hueValue.value,saturationValue.value,brightnessValue.value);const length=(vertical?height.value:width.value)-(boundedThumb?thumbSize:0),percent=b/255*length,pos=(reverse?length-percent:percent)-(boundedThumb?0:thumbSize/2),posY=vertical?pos:height.value/2-thumbSize/2,posX=vertical?width.value/2-thumbSize/2:pos;return{transform:[{translateY:posY},{translateX:posX},{scale:handleScale.value}]};}}';
      _f.__workletHash = 17330151530489;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\RGB\\BlueSlider.tsx (60:39)';
      _f.__optimalization = 2;
      return _f;
    })(),
    [localThumbSize, vertical, reverse]
  );
  const onGestureUpdate = (function () {
    const _f = function (_ref2) {
      let { x, y } = _ref2;
      const { r, g, b } = (0, _utils.HSVA2RGBA)(hueValue.value, saturationValue.value, brightnessValue.value);
      const length = (vertical ? height.value : width.value) - (boundedThumb ? thumbSize : 0),
        pos = (0, _utils.clamp)((vertical ? y : x) - (boundedThumb ? thumbSize / 2 : 0), length),
        value = Math.round((pos / length) * 255),
        newBlueValue = reverse ? 255 - value : value;
      if (newBlueValue === b) return;
      const { h, s, v } = (0, _utils.RGBA2HSVA)(r, g, newBlueValue);
      hueValue.value = h;
      saturationValue.value = s;
      brightnessValue.value = v;
      (0, _reactNativeReanimated.runOnJS)(onGestureChange)();
    };
    _f._closure = {
      HSVA2RGBA: _utils.HSVA2RGBA,
      hueValue,
      saturationValue,
      brightnessValue,
      vertical,
      height,
      width,
      boundedThumb,
      thumbSize,
      clamp: _utils.clamp,
      reverse,
      RGBA2HSVA: _utils.RGBA2HSVA,
      runOnJS: _reactNativeReanimated.runOnJS,
      onGestureChange,
    };
    _f.asString =
      'function _f({x:x,y:y}){const{HSVA2RGBA,hueValue,saturationValue,brightnessValue,vertical,height,width,boundedThumb,thumbSize,clamp,reverse,RGBA2HSVA,runOnJS,onGestureChange}=jsThis._closure;{const{r:r,g:g,b:b}=HSVA2RGBA(hueValue.value,saturationValue.value,brightnessValue.value);const length=(vertical?height.value:width.value)-(boundedThumb?thumbSize:0),pos=clamp((vertical?y:x)-(boundedThumb?thumbSize/2:0),length),value=Math.round(pos/length*255),newBlueValue=reverse?255-value:value;if(newBlueValue===b)return;const{h:h,s:s,v:v}=RGBA2HSVA(r,g,newBlueValue);hueValue.value=h;saturationValue.value=s;brightnessValue.value=v;runOnJS(onGestureChange)();}}';
    _f.__workletHash = 17463883460939;
    _f.__location =
      'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\RGB\\BlueSlider.tsx (73:26)';
    return _f;
  })();
  const onGestureBegin = (function () {
    const _f = function (event) {
      handleScale.value = (0, _reactNativeReanimated.withTiming)(1.2, {
        duration: 100,
      });
      onGestureUpdate(event);
    };
    _f._closure = {
      handleScale,
      withTiming: _reactNativeReanimated.withTiming,
      onGestureUpdate,
    };
    _f.asString =
      'function _f(event){const{handleScale,withTiming,onGestureUpdate}=jsThis._closure;{handleScale.value=withTiming(1.2,{duration:100});onGestureUpdate(event);}}';
    _f.__workletHash = 14773240202957;
    _f.__location =
      'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\RGB\\BlueSlider.tsx (93:25)';
    return _f;
  })();
  const onGestureFinish = (function () {
    const _f = function () {
      handleScale.value = (0, _reactNativeReanimated.withTiming)(1, {
        duration: 100,
      });
      (0, _reactNativeReanimated.runOnJS)(onGestureEnd)();
    };
    _f._closure = {
      handleScale,
      withTiming: _reactNativeReanimated.withTiming,
      runOnJS: _reactNativeReanimated.runOnJS,
      onGestureEnd,
    };
    _f.asString =
      'function _f(){const{handleScale,withTiming,runOnJS,onGestureEnd}=jsThis._closure;{handleScale.value=withTiming(1,{duration:100});runOnJS(onGestureEnd)();}}';
    _f.__workletHash = 6379049506140;
    _f.__location =
      'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\RGB\\BlueSlider.tsx (98:26)';
    return _f;
  })();
  const pan = _reactNativeGestureHandler.Gesture.Pan().onBegin(onGestureBegin).onUpdate(onGestureUpdate).onEnd(onGestureFinish);
  const tap = _reactNativeGestureHandler.Gesture.Tap().onTouchesUp(onGestureFinish);
  const longPress = _reactNativeGestureHandler.Gesture.LongPress().onTouchesUp(onGestureFinish);
  const composed = _reactNativeGestureHandler.Gesture.Exclusive(pan, tap, longPress);
  const onLayout = _ref3 => {
    let {
      nativeEvent: { layout },
    } = _ref3;
    if (!vertical)
      width.value = (0, _reactNativeReanimated.withTiming)(layout.width, {
        duration: 5,
      });
    if (vertical)
      height.value = (0, _reactNativeReanimated.withTiming)(layout.height, {
        duration: 5,
      });
  };
  const redGreen = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        const { r, g } = (0, _utils.HSVA2RGBA)(hueValue.value, saturationValue.value, brightnessValue.value);
        if (_utils.isWeb) {
          const deg = vertical ? (reverse ? 180 : 0) : reverse ? 90 : 270;
          return {
            background: `linear-gradient(${deg}deg, rgb(${r}, ${g}, 255) 0%, rgb(${r}, ${g}, 0) 100%)`,
          };
        }
        return {
          backgroundColor: `rgb(${r}, ${g}, 0)`,
        };
      };
      _f._closure = {
        HSVA2RGBA: _utils.HSVA2RGBA,
        hueValue,
        saturationValue,
        brightnessValue,
        isWeb: _utils.isWeb,
        vertical,
        reverse,
      };
      _f.asString =
        'function _f(){const{HSVA2RGBA,hueValue,saturationValue,brightnessValue,isWeb,vertical,reverse}=jsThis._closure;{const{r:r,g:g}=HSVA2RGBA(hueValue.value,saturationValue.value,brightnessValue.value);if(isWeb){const deg=vertical?reverse?180:0:reverse?90:270;return{background:"linear-gradient("+deg+"deg, rgb("+r+", "+g+", 255) 0%, rgb("+r+", "+g+", 0) 100%)"};}return{backgroundColor:"rgb("+r+", "+g+", 0)"};}}';
      _f.__workletHash = 8654328971661;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\RGB\\BlueSlider.tsx (114:36)';
      return _f;
    })()
  );
  const imageStyle = (0, _reactNativeReanimated.useAnimatedStyle)(
    (function () {
      const _f = function () {
        if (_utils.isWeb) return {};
        const imageRotate = vertical ? (reverse ? '90deg' : '270deg') : reverse ? '0deg' : '180deg';
        const imageTranslateY =
          ((height.value - width.value) / 2) * ((reverse && _utils.isRtl) || (!reverse && !_utils.isRtl) ? -1 : 1);
        const { r, g } = (0, _utils.HSVA2RGBA)(hueValue.value, saturationValue.value, brightnessValue.value);
        return {
          tintColor: `rgb(${r}, ${g}, 255)`,
          width: vertical ? height.value : '100%',
          height: vertical ? width.value : '100%',
          borderRadius,
          transform: [
            {
              rotate: imageRotate,
            },
            {
              translateX: vertical ? ((height.value - width.value) / 2) * (reverse ? 1 : -1) : 0,
            },
            {
              translateY: vertical ? imageTranslateY : 0,
            },
          ],
        };
      };
      _f._closure = {
        isWeb: _utils.isWeb,
        vertical,
        reverse,
        height,
        width,
        isRtl: _utils.isRtl,
        HSVA2RGBA: _utils.HSVA2RGBA,
        hueValue,
        saturationValue,
        brightnessValue,
        borderRadius,
      };
      _f.asString =
        "function _f(){const{isWeb,vertical,reverse,height,width,isRtl,HSVA2RGBA,hueValue,saturationValue,brightnessValue,borderRadius}=jsThis._closure;{if(isWeb)return{};const imageRotate=vertical?reverse?'90deg':'270deg':reverse?'0deg':'180deg';const imageTranslateY=(height.value-width.value)/2*(reverse&&isRtl||!reverse&&!isRtl?-1:1);const{r:r,g:g}=HSVA2RGBA(hueValue.value,saturationValue.value,brightnessValue.value);return{tintColor:\"rgb(\"+r+\", \"+g+\", 255)\",width:vertical?height.value:'100%',height:vertical?width.value:'100%',borderRadius:borderRadius,transform:[{rotate:imageRotate},{translateX:vertical?(height.value-width.value)/2*(reverse?1:-1):0},{translateY:vertical?imageTranslateY:0}]};}}";
      _f.__workletHash = 4122179154934;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Sliders\\RGB\\BlueSlider.tsx (123:38)';
      return _f;
    })(),
    [vertical, reverse, sliderThickness]
  );
  const thicknessStyle = vertical
    ? {
        width: sliderThickness,
      }
    : {
        height: sliderThickness,
      };
  return /*#__PURE__*/ _react.default.createElement(
    _reactNativeGestureHandler.GestureDetector,
    {
      gesture: composed,
    },
    /*#__PURE__*/ _react.default.createElement(
      _reactNativeReanimated.default.View,
      {
        onLayout: onLayout,
        style: [
          {
            borderRadius,
          },
          style,
          {
            position: 'relative',
            borderWidth: 0,
            padding: 0,
          },
          thicknessStyle,
          redGreen,
        ],
      },
      /*#__PURE__*/ _react.default.createElement(
        _utils.RenderNativeOnly,
        null,
        /*#__PURE__*/ _react.default.createElement(_reactNativeReanimated.default.Image, {
          source: require('../../../assets/blackGradient.png'),
          style: imageStyle,
        })
      ),
      /*#__PURE__*/ _react.default.createElement(_Thumb.default, {
        thumbShape,
        thumbSize,
        thumbColor,
        renderThumb,
        handleStyle,
        innerStyle: thumbInnerStyle,
        style: thumbStyle,
        vertical,
      })
    )
  );
}
//# sourceMappingURL=BlueSlider.js.map
