'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true,
});
exports.ConditionalRendering = ConditionalRendering;
exports.RGBA2HSVA = exports.HSVA2RGBA = exports.HSVA2HSLA = void 0;
exports.RenderNativeOnly = RenderNativeOnly;
exports.RenderWebOnly = RenderWebOnly;
exports.decimalToHex = exports.clamp = void 0;
exports.getStyle = getStyle;
exports.isWeb = exports.isRtl = void 0;
var _react = _interopRequireDefault(require('react'));
var _reactNative = require('react-native');
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}
/** - Get a specific property from a react native style object */
function getStyle(style, property) {
  const flattened = _reactNative.StyleSheet.flatten(style);
  return flattened[property];
}

/** - Clamp a number value between `0` and a max value */
const clamp = (function () {
  const _f = function (v, max) {
    return Math.min(Math.max(v, 0), max);
  };
  _f._closure = {};
  _f.asString = 'function clamp(v,max){return Math.min(Math.max(v,0),max);}';
  _f.__workletHash = 7369916205602;
  _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\utils.tsx (13:7)';
  return _f;
})();
/** - Convert `HSV` color to an `HSLA` string representation */
exports.clamp = clamp;
const HSVA2HSLA = (function () {
  const _f = function (h, s, v) {
    let a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;
    s = s / 100;
    v = v / 100;
    const l = ((2 - s) * v) / 2,
      sl = s * v,
      sln = l !== 0 && l !== 1 ? sl / (l < 0.5 ? l * 2 : 2 - l * 2) : sl;
    return `hsla(${h}, ${sln * 100}%, ${l * 100}%, ${a})`;
  };
  _f._closure = {};
  _f.asString =
    'function HSVA2HSLA(h,s,v,a=1){s=s/100;v=v/100;const l=(2-s)*v/2,sl=s*v,sln=l!==0&&l!==1?sl/(l<0.5?l*2:2-l*2):sl;return"hsla("+h+", "+sln*100+"%, "+l*100+"%, "+a+")";}';
  _f.__workletHash = 10633345248113;
  _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\utils.tsx (19:7)';
  return _f;
})();
/** - Convert `HSV` color to an `RGBA` object representation */
exports.HSVA2HSLA = HSVA2HSLA;
const HSVA2RGBA = (function () {
  const _f = function (h, s, v) {
    let a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;
    h = h / 360;
    s = s / 100;
    v = v / 100;
    const i = Math.floor(h * 6),
      f = h * 6 - i,
      p = v * (1 - s),
      q = v * (1 - f * s),
      t = v * (1 - (1 - f) * s);
    let r = 0,
      g = 0,
      b = 0;
    switch (i % 6) {
      case 0:
        r = v;
        g = t;
        b = p;
        break;
      case 1:
        r = q;
        g = v;
        b = p;
        break;
      case 2:
        r = p;
        g = v;
        b = t;
        break;
      case 3:
        r = p;
        g = q;
        b = v;
        break;
      case 4:
        r = t;
        g = p;
        b = v;
        break;
      case 5:
        r = v;
        g = p;
        b = q;
        break;
    }
    return {
      r: clamp(r * 255, 255),
      g: clamp(g * 255, 255),
      b: clamp(b * 255, 255),
      a: clamp(a, 1),
    };
  };
  _f._closure = {
    clamp,
  };
  _f.asString =
    'function HSVA2RGBA(h,s,v,a=1){const{clamp}=jsThis._closure;{h=h/360;s=s/100;v=v/100;const i=Math.floor(h*6),f=h*6-i,p=v*(1-s),q=v*(1-f*s),t=v*(1-(1-f)*s);let r=0,g=0,b=0;switch(i%6){case 0:r=v;g=t;b=p;break;case 1:r=q;g=v;b=p;break;case 2:r=p;g=v;b=t;break;case 3:r=p;g=q;b=v;break;case 4:r=t;g=p;b=v;break;case 5:r=v;g=p;b=q;break;}return{r:clamp(r*255,255),g:clamp(g*255,255),b:clamp(b*255,255),a:clamp(a,1)};}}';
  _f.__workletHash = 1438897234743;
  _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\utils.tsx (33:7)';
  return _f;
})();
/** - Convert `RGBA` color to an `HSVA`  object representation */
exports.HSVA2RGBA = HSVA2RGBA;
const RGBA2HSVA = (function () {
  const _f = function (r, g, b) {
    let a = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;
    r = r / 255;
    g = g / 255;
    b = b / 255;
    const max = Math.max(r, g, b),
      min = Math.min(r, g, b),
      d = max - min;
    let v = max,
      h = 0,
      s = max === 0 ? 0 : d / max;
    if (max === min) {
      h = 0;
    } else {
      if (max === r) {
        h = (g - b) / d + (g < b ? 6 : 0);
      } else if (max === g) {
        h = (b - r) / d + 2;
      } else if (max === b) {
        h = (r - g) / d + 4;
      }
      h = h / 6;
    }
    return {
      h: clamp(h * 360, 360),
      s: clamp(s * 100, 100),
      v: clamp(v * 100, 100),
      a: clamp(a, 1),
    };
  };
  _f._closure = {
    clamp,
  };
  _f.asString =
    'function RGBA2HSVA(r,g,b,a=1){const{clamp}=jsThis._closure;{r=r/255;g=g/255;b=b/255;const max=Math.max(r,g,b),min=Math.min(r,g,b),d=max-min;let v=max,h=0,s=max===0?0:d/max;if(max===min){h=0;}else{if(max===r){h=(g-b)/d+(g<b?6:0);}else if(max===g){h=(b-r)/d+2;}else if(max===b){h=(r-g)/d+4;}h=h/6;}return{h:clamp(h*360,360),s:clamp(s*100,100),v:clamp(v*100,100),a:clamp(a,1)};}}';
  _f.__workletHash = 14632142706471;
  _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\utils.tsx (91:7)';
  return _f;
})();
/** - Convert a decimal number to hexadecimal */
exports.RGBA2HSVA = RGBA2HSVA;
const decimalToHex = (function () {
  const _f = function (a) {
    a = clamp(Math.round(a * 255), 255);
    const hex = a.toString(16);
    return hex.length === 1 ? '0' + hex : hex;
  };
  _f._closure = {
    clamp,
  };
  _f.asString =
    "function decimalToHex(a){const{clamp}=jsThis._closure;{a=clamp(Math.round(a*255),255);const hex=a.toString(16);return hex.length===1?'0'+hex:hex;}}";
  _f.__workletHash = 9174819285907;
  _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\utils.tsx (128:7)';
  return _f;
})();
/** - Render children only if the `render` property is `true` */
exports.decimalToHex = decimalToHex;
function ConditionalRendering(_ref) {
  let { children, render } = _ref;
  if (!render) return null;
  return /*#__PURE__*/ _react.default.createElement(_react.default.Fragment, null, children);
}

/** - Render children for native platforms only (Android, IOS) */
function RenderNativeOnly(_ref2) {
  let { children } = _ref2;
  if (isWeb) return null;
  return /*#__PURE__*/ _react.default.createElement(_react.default.Fragment, null, children);
}

/** - Render children for Web platform only */
function RenderWebOnly(_ref3) {
  let { children } = _ref3;
  if (!isWeb) return null;
  return /*#__PURE__*/ _react.default.createElement(_react.default.Fragment, null, children);
}
const isRtl = _reactNative.I18nManager.isRTL;
exports.isRtl = isRtl;
const isWeb = _reactNative.Platform.OS === 'web';
exports.isWeb = isWeb;
//# sourceMappingURL=utils.js.map
