import React, { useRef, useState } from 'react';
import { runOnJS, useDerivedValue } from 'react-native-reanimated';
import WidgetTextInput from './WidgetTextInput';
import colorKit from '../../../colorKit/colorKit';
import { clamp } from '../../../utils';
export default function HslWidget(_ref) {
  let {
    onChange,
    returnedResults,
    hueValue,
    saturationValue,
    brightnessValue,
    alphaValue,
    inputStyle,
    inputTitleStyle,
    inputProps,
  } = _ref;
  const [hslValues, setHslValues] = useState(colorKit.HSL(returnedResults().hsla).object());
  const isFocused = useRef(false);
  const updateText = () => {
    const { h, s, l, a } = colorKit.HSL(returnedResults().hsla).object();
    if (!isFocused.current)
      setHslValues({
        h,
        s,
        l,
        a,
      });
  };
  useDerivedValue(
    (function () {
      const _f = function () {
        [hueValue.value, saturationValue.value, brightnessValue.value, alphaValue.value]; // To track changes
        runOnJS(updateText)();
      };
      _f._closure = {
        hueValue,
        saturationValue,
        brightnessValue,
        alphaValue,
        runOnJS,
        updateText,
      };
      _f.asString =
        'function _f(){const{hueValue,saturationValue,brightnessValue,alphaValue,runOnJS,updateText}=jsThis._closure;{[hueValue.value,saturationValue.value,brightnessValue.value,alphaValue.value];runOnJS(updateText)();}}';
      _f.__workletHash = 8119592659489;
      _f.__location =
        'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\InputWidget\\Widgets\\HslWidget.tsx (30:18)';
      return _f;
    })(),
    []
  );
  const onHueChange = text => {
    let hue = +text;
    hue = clamp(isNaN(hue) ? 0 : hue, 360);
    setHslValues(prev => ({
      ...prev,
      h: hue,
    }));
    onChange(`hsla(${hue}, ${hslValues.s}%, ${hslValues.l}%, ${hslValues.a})`);
  };
  const onSaturationChange = text => {
    let saturation = +text;
    saturation = clamp(isNaN(saturation) ? 0 : saturation, 100);
    setHslValues(prev => ({
      ...prev,
      s: saturation,
    }));
    onChange(`hsla(${hslValues.h}, ${saturation}%, ${hslValues.l}%, ${hslValues.a})`);
  };
  const onLumChange = text => {
    let lum = +text;
    lum = clamp(isNaN(lum) ? 0 : lum, 100);
    setHslValues(prev => ({
      ...prev,
      l: lum,
    }));
    onChange(`hsla(${hslValues.h}, ${hslValues.s}%, ${lum}%, ${hslValues.a})`);
  };
  const onAlphaChange = text => {
    let alpha = parseFloat(text);
    alpha = clamp(isNaN(alpha) ? 0 : alpha, 1);
    setHslValues(prev => ({
      ...prev,
      a: alpha,
    }));
    onChange(`hsla(${hslValues.h}, ${hslValues.s}%, ${hslValues.l}%, ${alpha})`);
  };
  const onFocus = () => {
    isFocused.current = true;
  };
  const onBlur = () => {
    isFocused.current = false;
  };
  return /*#__PURE__*/ React.createElement(
    React.Fragment,
    null,
    /*#__PURE__*/ React.createElement(WidgetTextInput, {
      inputStyle: inputStyle,
      textStyle: inputTitleStyle,
      value: hslValues.h,
      title: 'H',
      onChange: onHueChange,
      onBlur: onBlur,
      onFocus: onFocus,
      inputProps: inputProps,
    }),
    /*#__PURE__*/ React.createElement(WidgetTextInput, {
      inputStyle: inputStyle,
      textStyle: inputTitleStyle,
      value: hslValues.s,
      title: 'S',
      onChange: onSaturationChange,
      onBlur: onBlur,
      onFocus: onFocus,
      inputProps: inputProps,
    }),
    /*#__PURE__*/ React.createElement(WidgetTextInput, {
      inputStyle: inputStyle,
      textStyle: inputTitleStyle,
      value: hslValues.l,
      title: 'L',
      onChange: onLumChange,
      onBlur: onBlur,
      onFocus: onFocus,
      inputProps: inputProps,
    }),
    /*#__PURE__*/ React.createElement(WidgetTextInput, {
      inputStyle: inputStyle,
      textStyle: inputTitleStyle,
      value: hslValues.a,
      title: 'A',
      onChange: onAlphaChange,
      onBlur: onBlur,
      onFocus: onFocus,
      inputProps: inputProps,
      decimal: true,
    })
  );
}
//# sourceMappingURL=HslWidget.js.map
