import React, { useContext, useCallback } from 'react';
import { ImageBackground, StyleSheet } from 'react-native';
import { GestureDetector, Gesture } from 'react-native-gesture-handler';
import Animated, { runOnJS, useAnimatedStyle, useSharedValue, withTiming } from 'react-native-reanimated';
import { styles } from '../../styles';
import pickerContext from '../../AppContext';
import { clamp, ConditionalRendering, getStyle, HSVA2HSLA, isRtl } from '../../utils';
import Thumb from '../Thumb/Thumb';
export function Panel2(_ref) {
  let {
    adaptSpectrum: localAdaptSpectrum,
    thumbColor: localThumbColor,
    boundedThumb: localBoundedThumb,
    renderThumb: localRenderThumb,
    thumbShape: localThumbShape,
    thumbSize: localThumbSize,
    thumbStyle: localThumbStyle,
    thumbInnerStyle: localThumbInnerStyle,
    verticalChannel = 'saturation',
    reverseHue = false,
    reverseVerticalChannel = false,
    style = {},
  } = _ref;
  const {
    hueValue,
    saturationValue,
    brightnessValue,
    onGestureChange,
    onGestureEnd,
    adaptSpectrum: globalAdaptSpectrum,
    thumbSize: globalThumbsSize,
    thumbShape: globalThumbShape,
    thumbColor: globalThumbsColor,
    boundedThumb: globalBoundedThumb,
    renderThumb: globalRenderThumbs,
    thumbStyle: globalThumbsStyle,
    thumbInnerStyle: globalThumbsInnerStyle,
  } = useContext(pickerContext);
  const thumbShape = localThumbShape ?? globalThumbShape,
    thumbSize = localThumbSize ?? globalThumbsSize,
    thumbColor = localThumbColor ?? globalThumbsColor,
    boundedThumb = localBoundedThumb ?? globalBoundedThumb,
    renderThumb = localRenderThumb ?? globalRenderThumbs,
    thumbStyle = localThumbStyle ?? globalThumbsStyle ?? {},
    thumbInnerStyle = localThumbInnerStyle ?? globalThumbsInnerStyle ?? {},
    adaptSpectrum = localAdaptSpectrum ?? globalAdaptSpectrum,
    channelValue = verticalChannel === 'brightness' ? brightnessValue : saturationValue;
  const borderRadius = getStyle(style, 'borderRadius') ?? 5,
    getHeight = getStyle(style, 'height') ?? 200;
  const width = useSharedValue(0),
    height = useSharedValue(0);
  const handleScale = useSharedValue(1);
  const handleStyle = useAnimatedStyle(
    (function () {
      const _f = function () {
        const length = {
            x: width.value - (boundedThumb ? thumbSize : 0),
            y: height.value - (boundedThumb ? thumbSize : 0),
          },
          percentX = (hueValue.value / 360) * length.x,
          posX = (reverseHue ? length.x - percentX : percentX) - (boundedThumb ? 0 : thumbSize / 2),
          percentY = (channelValue.value / 100) * length.y,
          posY = (reverseVerticalChannel ? percentY : length.y - percentY) - (boundedThumb ? 0 : thumbSize / 2);
        return {
          transform: [
            {
              translateX: posX,
            },
            {
              translateY: posY,
            },
            {
              scale: handleScale.value,
            },
          ],
        };
      };
      _f._closure = {
        width,
        boundedThumb,
        thumbSize,
        height,
        hueValue,
        reverseHue,
        channelValue,
        reverseVerticalChannel,
        handleScale,
      };
      _f.asString =
        'function _f(){const{width,boundedThumb,thumbSize,height,hueValue,reverseHue,channelValue,reverseVerticalChannel,handleScale}=jsThis._closure;{const length={x:width.value-(boundedThumb?thumbSize:0),y:height.value-(boundedThumb?thumbSize:0)},percentX=hueValue.value/360*length.x,posX=(reverseHue?length.x-percentX:percentX)-(boundedThumb?0:thumbSize/2),percentY=channelValue.value/100*length.y,posY=(reverseVerticalChannel?percentY:length.y-percentY)-(boundedThumb?0:thumbSize/2);return{transform:[{translateX:posX},{translateY:posY},{scale:handleScale.value}]};}}';
      _f.__workletHash = 17565425960519;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel2.tsx (63:39)';
      _f.__optimalization = 3;
      return _f;
    })(),
    [localThumbSize, reverseHue, reverseVerticalChannel]
  );
  const spectrumStyle = useAnimatedStyle(
    (function () {
      const _f = function () {
        if (!adaptSpectrum) return {};
        if (verticalChannel === 'brightness')
          return {
            backgroundColor: HSVA2HSLA(0, 0, 100, 1 - saturationValue.value / 100),
          };
        return {
          backgroundColor: HSVA2HSLA(0, 0, 0, 1 - brightnessValue.value / 100),
        };
      };
      _f._closure = {
        adaptSpectrum,
        verticalChannel,
        HSVA2HSLA,
        saturationValue,
        brightnessValue,
      };
      _f.asString =
        "function _f(){const{adaptSpectrum,verticalChannel,HSVA2HSLA,saturationValue,brightnessValue}=jsThis._closure;{if(!adaptSpectrum)return{};if(verticalChannel==='brightness')return{backgroundColor:HSVA2HSLA(0,0,100,1-saturationValue.value/100)};return{backgroundColor:HSVA2HSLA(0,0,0,1-brightnessValue.value/100)};}}";
      _f.__workletHash = 751007878060;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel2.tsx (72:41)';
      return _f;
    })()
  );
  const onGestureUpdate = (function () {
    const _f = function (_ref2) {
      let { x, y } = _ref2;
      const lengthX = width.value - (boundedThumb ? thumbSize : 0),
        lengthY = height.value - (boundedThumb ? thumbSize : 0),
        posX = clamp(x - (boundedThumb ? thumbSize / 2 : 0), lengthX),
        posY = clamp(y - (boundedThumb ? thumbSize / 2 : 0), lengthY),
        valueX = Math.round((posX / lengthX) * 360),
        valueY = Math.round((posY / lengthY) * 100),
        newHueValue = reverseHue ? 360 - valueX : valueX,
        newChannelValue = reverseVerticalChannel ? valueY : 100 - valueY;
      if (hueValue.value === newHueValue && channelValue.value === newChannelValue) return;
      hueValue.value = newHueValue;
      channelValue.value = newChannelValue;
      runOnJS(onGestureChange)();
    };
    _f._closure = {
      width,
      boundedThumb,
      thumbSize,
      height,
      clamp,
      reverseHue,
      reverseVerticalChannel,
      hueValue,
      channelValue,
      runOnJS,
      onGestureChange,
    };
    _f.asString =
      'function _f({x:x,y:y}){const{width,boundedThumb,thumbSize,height,clamp,reverseHue,reverseVerticalChannel,hueValue,channelValue,runOnJS,onGestureChange}=jsThis._closure;{const lengthX=width.value-(boundedThumb?thumbSize:0),lengthY=height.value-(boundedThumb?thumbSize:0),posX=clamp(x-(boundedThumb?thumbSize/2:0),lengthX),posY=clamp(y-(boundedThumb?thumbSize/2:0),lengthY),valueX=Math.round(posX/lengthX*360),valueY=Math.round(posY/lengthY*100),newHueValue=reverseHue?360-valueX:valueX,newChannelValue=reverseVerticalChannel?valueY:100-valueY;if(hueValue.value===newHueValue&&channelValue.value===newChannelValue)return;hueValue.value=newHueValue;channelValue.value=newChannelValue;runOnJS(onGestureChange)();}}';
    _f.__workletHash = 1354008168735;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel2.tsx (78:26)';
    return _f;
  })();
  const onGestureBegin = (function () {
    const _f = function (event) {
      handleScale.value = withTiming(1.2, {
        duration: 100,
      });
      onGestureUpdate(event);
    };
    _f._closure = {
      handleScale,
      withTiming,
      onGestureUpdate,
    };
    _f.asString =
      'function _f(event){const{handleScale,withTiming,onGestureUpdate}=jsThis._closure;{handleScale.value=withTiming(1.2,{duration:100});onGestureUpdate(event);}}';
    _f.__workletHash = 14773240202957;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel2.tsx (96:25)';
    return _f;
  })();
  const onGestureFinish = (function () {
    const _f = function () {
      handleScale.value = withTiming(1, {
        duration: 100,
      });
      runOnJS(onGestureEnd)();
    };
    _f._closure = {
      handleScale,
      withTiming,
      runOnJS,
      onGestureEnd,
    };
    _f.asString =
      'function _f(){const{handleScale,withTiming,runOnJS,onGestureEnd}=jsThis._closure;{handleScale.value=withTiming(1,{duration:100});runOnJS(onGestureEnd)();}}';
    _f.__workletHash = 6379049506140;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel2.tsx (101:26)';
    return _f;
  })();
  const pan = Gesture.Pan().onBegin(onGestureBegin).onUpdate(onGestureUpdate).onEnd(onGestureFinish);
  const tap = Gesture.Tap().onTouchesUp(onGestureFinish);
  const longPress = Gesture.LongPress().onTouchesUp(onGestureFinish);
  const composed = Gesture.Exclusive(pan, tap, longPress);
  const onLayout = useCallback(_ref3 => {
    let {
      nativeEvent: { layout },
    } = _ref3;
    width.value = layout.width;
    height.value = layout.height;
  }, []);
  const rotatePanelImage = useAnimatedStyle(
    (function () {
      const _f = () => ({
        width: height.value,
        height: width.value,
        transform: [
          {
            rotate: reverseVerticalChannel ? '90deg' : '270deg',
          },
          {
            translateX: ((width.value - height.value) / 2) * (reverseVerticalChannel ? -1 : 1),
          },
          {
            translateY: ((width.value - height.value) / 2) * (isRtl ? -1 : 1) * (reverseVerticalChannel ? -1 : 1),
          },
        ],
      });
      _f._closure = {
        height,
        width,
        reverseVerticalChannel,
        isRtl,
      };
      _f.asString =
        "function _f(){const{height,width,reverseVerticalChannel,isRtl}=jsThis._closure;{return{width:height.value,height:width.value,transform:[{rotate:reverseVerticalChannel?'90deg':'270deg'},{translateX:(width.value-height.value)/2*(reverseVerticalChannel?-1:1)},{translateY:(width.value-height.value)/2*(isRtl?-1:1)*(reverseVerticalChannel?-1:1)}]};}}";
      _f.__workletHash = 2093867738315;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel2.tsx (117:44)';
      _f.__optimalization = 3;
      return _f;
    })()
  );
  return /*#__PURE__*/ React.createElement(
    GestureDetector,
    {
      gesture: composed,
    },
    /*#__PURE__*/ React.createElement(
      Animated.View,
      {
        onLayout: onLayout,
        style: [
          styles.panel_container,
          {
            height: getHeight,
          },
          style,
          {
            position: 'relative',
            borderWidth: 0,
            padding: 0,
          },
        ],
      },
      /*#__PURE__*/ React.createElement(
        ImageBackground,
        {
          source: require('../../assets/Hue.png'),
          style: [
            styles.panel_image,
            {
              position: 'relative',
              borderRadius,
              transform: [
                {
                  scaleX: reverseHue ? -1 : 1,
                },
              ],
            },
          ],
          resizeMode: 'stretch',
        },
        /*#__PURE__*/ React.createElement(
          ConditionalRendering,
          {
            render: adaptSpectrum && verticalChannel === 'brightness',
          },
          /*#__PURE__*/ React.createElement(Animated.View, {
            style: [spectrumStyle, StyleSheet.absoluteFillObject],
          })
        ),
        /*#__PURE__*/ React.createElement(Animated.Image, {
          source: require('../../assets/blackGradient.png'),
          style: [
            styles.panel_image,
            rotatePanelImage,
            {
              tintColor: verticalChannel === 'saturation' ? '#fff' : undefined,
            },
          ],
          resizeMode: 'stretch',
        }),
        /*#__PURE__*/ React.createElement(
          ConditionalRendering,
          {
            render: adaptSpectrum && verticalChannel === 'saturation',
          },
          /*#__PURE__*/ React.createElement(Animated.View, {
            style: [spectrumStyle, StyleSheet.absoluteFillObject],
          })
        )
      ),
      /*#__PURE__*/ React.createElement(Thumb, {
        channel: verticalChannel === 'brightness' ? 'v' : 's',
        thumbShape,
        thumbSize,
        thumbColor,
        renderThumb,
        innerStyle: thumbInnerStyle,
        style: thumbStyle,
        handleStyle,
        adaptSpectrum,
      })
    )
  );
}
//# sourceMappingURL=Panel2.js.map
