import React, { useContext, useCallback } from 'react';
import { ImageBackground, Image, StyleSheet } from 'react-native';
import { GestureDetector, Gesture } from 'react-native-gesture-handler';
import Animated, { runOnJS, useAnimatedStyle, useSharedValue, withTiming } from 'react-native-reanimated';
import { clamp, ConditionalRendering, HSVA2HSLA } from '../../utils';
import { styles } from '../../styles';
import pickerContext from '../../AppContext';
import Thumb from '../Thumb/Thumb';
export function Panel3(_ref) {
  let {
    adaptSpectrum: localAdaptSpectrum,
    thumbShape: localThumbShape,
    thumbSize: localThumbSize,
    thumbColor: localThumbColor,
    boundedThumb: localBoundedThumb,
    renderThumb: localRenderThumb,
    thumbStyle: localThumbStyle,
    thumbInnerStyle: localThumbInnerStyle,
    centerChannel = 'saturation',
    style = {},
  } = _ref;
  const {
    hueValue,
    saturationValue,
    brightnessValue,
    onGestureChange,
    onGestureEnd,
    adaptSpectrum: globalAdaptSpectrum,
    thumbSize: globalThumbsSize,
    thumbShape: globalThumbShape,
    thumbColor: globalThumbsColor,
    boundedThumb: globalBoundedThumb,
    renderThumb: globalRenderThumbs,
    thumbStyle: globalThumbsStyle,
    thumbInnerStyle: globalThumbsInnerStyle,
  } = useContext(pickerContext);
  const thumbShape = localThumbShape ?? globalThumbShape,
    thumbSize = localThumbSize ?? globalThumbsSize,
    thumbColor = localThumbColor ?? globalThumbsColor,
    boundedThumb = localBoundedThumb ?? globalBoundedThumb,
    renderThumb = localRenderThumb ?? globalRenderThumbs,
    thumbStyle = localThumbStyle ?? globalThumbsStyle ?? {},
    thumbInnerStyle = localThumbInnerStyle ?? globalThumbsInnerStyle ?? {},
    adaptSpectrum = localAdaptSpectrum ?? globalAdaptSpectrum,
    channelValue = centerChannel === 'brightness' ? brightnessValue : saturationValue;
  const borderRadius = 2000;
  const width = useSharedValue(0);
  const handleScale = useSharedValue(1);
  const handleStyle = useAnimatedStyle(
    (function () {
      const _f = function () {
        const center = width.value / 2 - (boundedThumb ? thumbSize / 2 : 0),
          distance = (channelValue.value / 100) * (width.value / 2 - (boundedThumb ? thumbSize / 2 : 0)),
          posY =
            width.value -
            Math.round(Math.sin((hueValue.value * Math.PI) / 180) * distance + center) -
            (boundedThumb ? thumbSize : thumbSize / 2),
          posX =
            width.value -
            Math.round(Math.cos((hueValue.value * Math.PI) / 180) * distance + center) -
            (boundedThumb ? thumbSize : thumbSize / 2);
        return {
          transform: [
            {
              translateX: posX,
            },
            {
              translateY: posY,
            },
            {
              scale: handleScale.value,
            },
            {
              rotate: hueValue.value + 90 + 'deg',
            },
          ],
        };
      };
      _f._closure = {
        width,
        boundedThumb,
        thumbSize,
        channelValue,
        hueValue,
        handleScale,
      };
      _f.asString =
        "function _f(){const{width,boundedThumb,thumbSize,channelValue,hueValue,handleScale}=jsThis._closure;{const center=width.value/2-(boundedThumb?thumbSize/2:0),distance=channelValue.value/100*(width.value/2-(boundedThumb?thumbSize/2:0)),posY=width.value-Math.round(Math.sin(hueValue.value*Math.PI/180)*distance+center)-(boundedThumb?thumbSize:thumbSize/2),posX=width.value-Math.round(Math.cos(hueValue.value*Math.PI/180)*distance+center)-(boundedThumb?thumbSize:thumbSize/2);return{transform:[{translateX:posX},{translateY:posY},{scale:handleScale.value},{rotate:hueValue.value+90+'deg'}]};}}";
      _f.__workletHash = 14589495593566;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel3.tsx (59:39)';
      _f.__optimalization = 2;
      return _f;
    })(),
    [localThumbSize]
  );
  const spectrumStyle = useAnimatedStyle(
    (function () {
      const _f = function () {
        if (!adaptSpectrum) return {};
        if (centerChannel === 'brightness')
          return {
            backgroundColor: HSVA2HSLA(0, 0, 100, 1 - saturationValue.value / 100),
          };
        return {
          backgroundColor: HSVA2HSLA(0, 0, 0, 1 - brightnessValue.value / 100),
        };
      };
      _f._closure = {
        adaptSpectrum,
        centerChannel,
        HSVA2HSLA,
        saturationValue,
        brightnessValue,
      };
      _f.asString =
        "function _f(){const{adaptSpectrum,centerChannel,HSVA2HSLA,saturationValue,brightnessValue}=jsThis._closure;{if(!adaptSpectrum)return{};if(centerChannel==='brightness')return{backgroundColor:HSVA2HSLA(0,0,100,1-saturationValue.value/100)};return{backgroundColor:HSVA2HSLA(0,0,0,1-brightnessValue.value/100)};}}";
      _f.__workletHash = 12419914282348;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel3.tsx (80:41)';
      return _f;
    })()
  );
  const onGestureUpdate = (function () {
    const _f = function (_ref2) {
      let { x, y } = _ref2;
      const center = (width.value - (boundedThumb ? thumbSize : 0)) / 2,
        dx = center - x + (boundedThumb ? thumbSize / 2 : 0),
        dy = center - y + (boundedThumb ? thumbSize / 2 : 0),
        radius = clamp(Math.sqrt(dx * dx + dy * dy), center),
        // distance from center
        theta = Math.atan2(dy, dx) * (180 / Math.PI),
        // [0 - 180] range
        angle = theta < 0 ? 360 + theta : theta,
        // [0 - 360] range
        radiusPercent = radius / center,
        newHueValue = Math.round(angle),
        newChannelValue = Math.round(radiusPercent * 100);
      if (hueValue.value === newHueValue && channelValue.value === newChannelValue) return;
      hueValue.value = newHueValue;
      channelValue.value = newChannelValue;
      runOnJS(onGestureChange)();
    };
    _f._closure = {
      width,
      boundedThumb,
      thumbSize,
      clamp,
      hueValue,
      channelValue,
      runOnJS,
      onGestureChange,
    };
    _f.asString =
      'function _f({x:x,y:y}){const{width,boundedThumb,thumbSize,clamp,hueValue,channelValue,runOnJS,onGestureChange}=jsThis._closure;{const center=(width.value-(boundedThumb?thumbSize:0))/2,dx=center-x+(boundedThumb?thumbSize/2:0),dy=center-y+(boundedThumb?thumbSize/2:0),radius=clamp(Math.sqrt(dx*dx+dy*dy),center),theta=Math.atan2(dy,dx)*(180/Math.PI),angle=theta<0?360+theta:theta,radiusPercent=radius/center,newHueValue=Math.round(angle),newChannelValue=Math.round(radiusPercent*100);if(hueValue.value===newHueValue&&channelValue.value===newChannelValue)return;hueValue.value=newHueValue;channelValue.value=newChannelValue;runOnJS(onGestureChange)();}}';
    _f.__workletHash = 15671765734771;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel3.tsx (86:26)';
    return _f;
  })();
  const onGestureBegin = (function () {
    const _f = function (event) {
      handleScale.value = withTiming(1.2, {
        duration: 100,
      });
      onGestureUpdate(event);
    };
    _f._closure = {
      handleScale,
      withTiming,
      onGestureUpdate,
    };
    _f.asString =
      'function _f(event){const{handleScale,withTiming,onGestureUpdate}=jsThis._closure;{handleScale.value=withTiming(1.2,{duration:100});onGestureUpdate(event);}}';
    _f.__workletHash = 14773240202957;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel3.tsx (105:25)';
    return _f;
  })();
  const onGestureFinish = (function () {
    const _f = function () {
      handleScale.value = withTiming(1, {
        duration: 100,
      });
      runOnJS(onGestureEnd)();
    };
    _f._closure = {
      handleScale,
      withTiming,
      runOnJS,
      onGestureEnd,
    };
    _f.asString =
      'function _f(){const{handleScale,withTiming,runOnJS,onGestureEnd}=jsThis._closure;{handleScale.value=withTiming(1,{duration:100});runOnJS(onGestureEnd)();}}';
    _f.__workletHash = 6379049506140;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Panels\\Panel3.tsx (110:26)';
    return _f;
  })();
  const pan = Gesture.Pan().onBegin(onGestureBegin).onUpdate(onGestureUpdate).onEnd(onGestureFinish);
  const tap = Gesture.Tap().onTouchesUp(onGestureFinish);
  const longPress = Gesture.LongPress().onTouchesUp(onGestureFinish);
  const composed = Gesture.Exclusive(pan, tap, longPress);
  const onLayout = useCallback(_ref3 => {
    let {
      nativeEvent: { layout },
    } = _ref3;
    const layoutWidth = layout.width;
    width.value = layoutWidth;
  }, []);
  return /*#__PURE__*/ React.createElement(
    GestureDetector,
    {
      gesture: composed,
    },
    /*#__PURE__*/ React.createElement(
      Animated.View,
      {
        onLayout: onLayout,
        style: [
          styles.panel_container,
          style,
          {
            position: 'relative',
            aspectRatio: 1,
            borderWidth: 0,
            padding: 0,
            borderRadius,
          },
        ],
      },
      /*#__PURE__*/ React.createElement(
        ImageBackground,
        {
          source: require('../../assets/circularHue.png'),
          style: styles.panel_image,
          resizeMode: 'stretch',
        },
        /*#__PURE__*/ React.createElement(
          ConditionalRendering,
          {
            render: adaptSpectrum && centerChannel === 'brightness',
          },
          /*#__PURE__*/ React.createElement(Animated.View, {
            style: [
              {
                borderRadius,
              },
              spectrumStyle,
              StyleSheet.absoluteFillObject,
            ],
          })
        ),
        /*#__PURE__*/ React.createElement(Image, {
          source: require('../../assets/blackRadial.png'),
          style: [
            styles.panel_image,
            {
              tintColor: centerChannel === 'saturation' ? '#fff' : undefined,
            },
          ],
          resizeMode: 'stretch',
        }),
        /*#__PURE__*/ React.createElement(
          ConditionalRendering,
          {
            render: adaptSpectrum && centerChannel === 'saturation',
          },
          /*#__PURE__*/ React.createElement(Animated.View, {
            style: [
              {
                borderRadius,
              },
              spectrumStyle,
              StyleSheet.absoluteFillObject,
            ],
          })
        )
      ),
      /*#__PURE__*/ React.createElement(Thumb, {
        channel: centerChannel === 'brightness' ? 'v' : 's',
        thumbShape,
        thumbSize,
        thumbColor,
        renderThumb,
        innerStyle: thumbInnerStyle,
        style: thumbStyle,
        handleStyle,
        adaptSpectrum,
      })
    )
  );
}
//# sourceMappingURL=Panel3.js.map
