import React, { useContext } from 'react';
import { runOnJS, useAnimatedStyle, useDerivedValue, useSharedValue } from 'react-native-reanimated';
import { styles } from '../../styles';
import colorKit from '../../colorKit/colorKit';
import pickerContext from '../../AppContext';
import BuiltinThumbs from './BuiltinThumbs/index';
export default function Thumb(_ref) {
  let {
    handleStyle,
    innerStyle,
    style,
    thumbColor,
    renderThumb: RenderThumb,
    thumbShape = 'ring',
    thumbSize,
    vertical = false,
    adaptSpectrum,
    channel,
  } = _ref;
  const { width, height, borderRadius } = {
    width: thumbSize,
    height: thumbSize,
    borderRadius: thumbSize / 2,
  };
  const { hueValue, saturationValue, brightnessValue, alphaValue, value } = useContext(pickerContext);
  const resultColor = useSharedValue('#ffffff');
  const solidColor = useAnimatedStyle(
    (function () {
      const _f = () => ({
        backgroundColor: resultColor.value,
      });
      _f._closure = {
        resultColor,
      };
      _f.asString = 'function _f(){const{resultColor}=jsThis._closure;{return{backgroundColor:resultColor.value};}}';
      _f.__workletHash = 12991829208188;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Thumb\\Thumb.tsx (27:38)';
      _f.__optimalization = 3;
      return _f;
    })()
  );
  const setResultColor = color => {
    resultColor.value = colorKit.HEX(color);
  };
  const adaptiveColor = useSharedValue('#ffffff');
  const setAdaptiveColor = color1 => {
    const color = adaptiveColor.value === '#ffffff' ? '#000000' : '#ffffff';
    const contrast = colorKit.contrastRatio(color1, adaptiveColor.value);
    adaptiveColor.value = contrast < 4.5 ? color : adaptiveColor.value;
  };

  /**
   * Get the current color and calculate its contrast ratio against white or black,
   * depending on the channel and whether 'adaptSpectrum' is enabled
   */
  const getColorForAdaptiveColor = (function () {
    const _f = function () {
      if (adaptSpectrum) {
        if (channel === 'a') {
          return alphaValue.value > 0.5
            ? {
                h: hueValue.value,
                s: saturationValue.value,
                v: brightnessValue.value,
              }
            : {
                h: 0,
                s: 0,
                v: 70,
              };
        }
        return {
          h: hueValue.value,
          s: saturationValue.value,
          v: brightnessValue.value,
        };
      }
      switch (channel) {
        case 'h':
          return {
            h: hueValue.value,
            s: 100,
            v: 100,
          };
        case 'v':
          return {
            h: hueValue.value,
            s: 100,
            v: brightnessValue.value,
          };
        case 's':
          return {
            h: hueValue.value,
            s: saturationValue.value,
            v: 70,
          };
        case 'a':
          return {
            h: hueValue.value,
            s: alphaValue.value * 100,
            v: 70,
          };
        default:
          return {
            h: hueValue.value,
            s: saturationValue.value,
            v: brightnessValue.value,
          };
      }
    };
    _f._closure = {
      adaptSpectrum,
      channel,
      alphaValue,
      hueValue,
      saturationValue,
      brightnessValue,
    };
    _f.asString =
      "function _f(){const{adaptSpectrum,channel,alphaValue,hueValue,saturationValue,brightnessValue}=jsThis._closure;{if(adaptSpectrum){if(channel==='a'){return alphaValue.value>0.5?{h:hueValue.value,s:saturationValue.value,v:brightnessValue.value}:{h:0,s:0,v:70};}return{h:hueValue.value,s:saturationValue.value,v:brightnessValue.value};}switch(channel){case'h':return{h:hueValue.value,s:100,v:100};case'v':return{h:hueValue.value,s:100,v:brightnessValue.value};case's':return{h:hueValue.value,s:saturationValue.value,v:70};case'a':return{h:hueValue.value,s:alphaValue.value*100,v:70};default:return{h:hueValue.value,s:saturationValue.value,v:brightnessValue.value};}}}";
    _f.__workletHash = 4227988470239;
    _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Thumb\\Thumb.tsx (43:35)';
    return _f;
  })();

  // When the values of channels change
  useDerivedValue(
    (function () {
      const _f = function () {
        alphaValue.value; // to keep track with `alphaValue` changes
        runOnJS(setAdaptiveColor)(getColorForAdaptiveColor());
        runOnJS(setResultColor)({
          h: hueValue.value,
          s: saturationValue.value,
          v: brightnessValue.value,
        });
      };
      _f._closure = {
        alphaValue,
        runOnJS,
        setAdaptiveColor,
        getColorForAdaptiveColor,
        setResultColor,
        hueValue,
        saturationValue,
        brightnessValue,
      };
      _f.asString =
        'function _f(){const{alphaValue,runOnJS,setAdaptiveColor,getColorForAdaptiveColor,setResultColor,hueValue,saturationValue,brightnessValue}=jsThis._closure;{alphaValue.value;runOnJS(setAdaptiveColor)(getColorForAdaptiveColor());runOnJS(setResultColor)({h:hueValue.value,s:saturationValue.value,v:brightnessValue.value});}}';
      _f.__workletHash = 12070766702289;
      _f.__location = 'D:\\Development\\--- Packages ---\\reanimated-color-picker\\src\\components\\Thumb\\Thumb.tsx (69:18)';
      return _f;
    })()
  );
  const thumbProps = {
    width,
    height,
    borderRadius,
    vertical,
    solidColor,
    adaptiveColor,
    handleStyle,
    innerStyle,
    style,
    thumbColor,
  };

  // render a custom thumb
  if (RenderThumb)
    return /*#__PURE__*/ React.createElement(RenderThumb, {
      positionStyle: [styles.handle, handleStyle],
      width: width,
      height: height,
      initialColor: value,
      currentColor: resultColor,
      adaptiveColor: adaptiveColor,
    });

  // normalize 'thumbShape' string to match 'BuiltinThumbs' keys.
  const thumb_Shape = thumbShape.toLowerCase().charAt(0).toUpperCase() + thumbShape.slice(1);
  if (thumb_Shape in BuiltinThumbs) {
    const SelectedThumb = BuiltinThumbs[thumb_Shape];
    return /*#__PURE__*/ React.createElement(SelectedThumb, thumbProps);
  }

  // default to the 'Ring' thumb
  return /*#__PURE__*/ React.createElement(BuiltinThumbs.Ring, thumbProps);
}
//# sourceMappingURL=Thumb.js.map
